;Simple Melody Player for ATTiny11(L)
;Fuse Byte: internal R/C osc. = 0x1C, ext. 1 MHz XTAM = 0x1F

;PORTB1: "Start" input (active low)
;PORTB0/2: Piezo Buzzer outputs
;PORTB3/4: Unused or external 1 MHz XTAL
;PORTB5: RESET', connect to Vcc

;         Piezo
;         |   |
; ,-----, | ,-)-----,
; |   ,-^-^-^-^-,   |
; |   > Tiny 11 |   :|<- push button (short for continuous playback)
; |   '-v-v-v-v-'   |
; |   1 |     |     |
; |     | | , |     |
; '-----o-| |-o-----'
;       + | ' Battery (3V - 5V)

;(C) Arne Rossius, http://www.ebps.de.vu/
;Version: 2007-07-14

.include "../include/tn11def.inc"

.def	lpmreg = R0
.def	backup = R1
.def	zero = R2
.def	out0 = R3
.def	out1 = R4
.def	out2 = R5

.def	temp = R16
.def	ratio = R17
.def	inttmp = R18
.def	step = R19
.def	intdiv = R20
.def	tonelen = R21
.def	length = R22

.def	regL = R26
.def	regH = R27
.def	lenL = R28
.def	lenH = R29
;R31:R30 = Z pointer (Flash pointer)

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org INT0addr
	reti ;only used for waking up from power down mode
	
;===============================================================================

;valid values: 5V: 8, 4.5V: 6, 3V: 2 (fast) or 3 (slow), ext. 1 MHz XTAL: 7
.equ SPEEDMUL = 2

notes:
;     c    c#     d    d#     e     f    f#     g    g#     a    a#     b
.if (SPEEDMUL == 8) ;5V -> ~1.1 MHz
.dw 2107, 1986, 1877, 1768, 1672, 1576, 1491, 1403, 1325, 1250, 1180, 1116
.dw 1054,  993,  939,  884,  836,  788,  745,  702,  663,  625,  590,  558
.dw  263,  248,  235,  221,  209,  197,  186,  175,  166,  156,  148,  139
.elif (SPEEDMUL == 7) ;ext. XTAL -> 1 MHz
.dw 1916, 1805, 1706, 1608, 1520, 1433, 1355, 1276, 1205, 1136, 1073, 1014
.dw  958,  903,  853,  804,  760,  716,  678,  638,  602,  568,  536,  507
.dw  239,  226,  213,  201,  190,  179,  169,  159,  151,  142,  134,  127
.elif (SPEEDMUL == 6) ;4.5V -> ~0.9 MHz
.dw 1724, 1625, 1536, 1447, 1368, 1289, 1220, 1148, 1084, 1023,  966,  913
.dw  862,  812,  768,  723,  684,  645,  610,  574,  542,  511,  483,  456
.dw  216,  203,  192,  181,  171,  161,  152,  143,  136,  128,  121,  114
.elif (SPEEDMUL <= 3) ;3V -> ~0.35 MHz
.dw  670,  632,  597,  563,  532,  501,  474,  446,  422,  398,  376,  355
.dw  335,  316,  299,  281,  266,  251,  237,  223,  211,  199,  188,  177
.dw   84,   79,   75,   70,   66,   63,   59,   56,   53,   50,   47,   44
.else
.error "Invalid SPEEDMUL value"
.endif
;octave 1
.equ	c1 = notes*2 + 0
.equ	cis1 = notes*2 + 2
.equ	d1 = notes*2 + 4
.equ	dis1 = notes*2 + 6
.equ	e1 = notes*2 + 8
.equ	f1 = notes*2 + 10
.equ	fis1 = notes*2 + 12
.equ	g1 = notes*2 + 14
.equ	gis1 = notes*2 + 16
.equ	a1 = notes*2 + 18
.equ	ais1 = notes*2 + 20
.equ	b1 = notes*2 + 22

;octave 2
.equ	c2 = notes*2 + 24
.equ	cis2 = notes*2 + 26
.equ	d2 = notes*2 + 28
.equ	dis2 = notes*2 + 30
.equ	e2 = notes*2 + 32
.equ	f2 = notes*2 + 34
.equ	fis2 = notes*2 + 36
.equ	g2 = notes*2 + 38
.equ	gis2 = notes*2 + 40
.equ	a2 = notes*2 + 42
.equ	ais2 = notes*2 + 44
.equ	b2 = notes*2 + 46

;octave 3
.equ	c3 = notes*2 + 48
.equ	cis3 = notes*2 + 50
.equ	d3 = notes*2 + 52
.equ	dis3 = notes*2 + 54
.equ	e3 = notes*2 + 56
.equ	f3 = notes*2 + 58
.equ	fis3 = notes*2 + 60
.equ	g3 = notes*2 + 62
.equ	gis3 = notes*2 + 64
.equ	a3 = notes*2 + 66
.equ	ais3 = notes*2 + 68
.equ	b3 = notes*2 + 70

;other constants
.equ	L = 0xE0 ;length (add 1..16 for 1/16..1/1 note)
.equ	LEGATO = 0xFA
.equ	NORMAL = 0xFB
.equ	STACCATO = 0xFC
.equ	P = 0xFE ;pause
.equ	END = 0xFF ;end of the melody

melody: ;note lengths: L|16 = whole, L|8 = half ... L|1 = sixteenth

;;"Alle meine Entchen"
;.equ	SPEED = 6 ;higher value = slower
;.db L|4, c1, d1, e1, f1, L|8, g1, g1
;;        Al--le  mei-ne      Ent-chen
;.db L|4, a1, a1, a1, a1, L|8, g1, P, L|4, a1, a1, a1, a1, L|8, g1, P
;;     Schwim-men auf dem     See  ,    Schwim-men auf dem     See  ,
;.db L|4, f1, f1, f1, f1, L|8, e1, e1, L|4, g1, g1, g1, g1, L|16, c1
;;    Koepf-chen un-ter      Was-ser, Schwaenz-chen in die      Hoeh'.
;.db L|16, P, P, END

;;Happy Birthday (modified)
;.equ	SPEED = 10
;.db L|3, d2, L|1, d2, L|4, e2, d2, g2, L|6, fis2, L|2, P
;.db L|3, d2, L|1, d2, L|4, e2, d2, a2, L|6, g2, L|2, P
;.db L|3, d2, L|1, d2, L|4, d3, b2, g2, fis2, L|6, e2, L|2, P
;.db L|3, c3, L|1, c3, L|4, b2, g2, a2, L|8, g2, P, END

;"Fuer Elise" (slightly modified):
.equ	SPEED = 12 ;higher value = slower
.db L|1, e2, dis2, e2, dis2, e2, b1, d2, c2, L|2, a1, L|1, P
.db c1, e1, a1, L|2, b1, L|1, P, e1, gis1, b1, L|2, c2, L|1, P
.db e1, e2, dis2, e2, dis2, e2, b1, d2, c2, L|2, a1, L|1, P
.db c1, e1, a1, L|2, b1, L|1, P, e1, c2, b1, L|2, a1, L|1, P
.db b1, c2, d2, L|3, e2, L|1, g1, f2, e2, L|3, d2, L|1
.db f1, e2, d2, L|3, c2, L|1, e1, d2, c2, L|2, b1, L|1, P
.db e1, e2, P, P, e2, e3, P, P, dis2, e2, P, P, dis2, e2, dis2
.db e2, dis2, e2, b1, d2, c2, L|2, a1, L|1, P
.db c1, e1, a1, L|2, b1, L|1, P, e1, gis1, b1, L|2, c2, L|1, P
.db e1, e2, dis2, e2, dis2, e2, b1, d2, c2, L|2, a1, L|1, P
.db c1, e1, a1, L|2, b1, L|1, P, e1, c2, b1, L|6, a1, P, END

;;Jingle Bells
;.equ	 SPEED = 8
;.db L|2, b1, b1, L|4, b1, L|2, b1, b1, L|4, b1, L|2, b1, d2, g1, a1, L|8, b1
;;       Jin- gle     bells,   Jin- gle     bells,   jin- gle all the     way.
;.db L|2, c2, c2, c2, c2, c2, b1, b1, L|1, b1, b1, L|2, b1, a1, a1, b1, L|6, a1, L|2, P
;;        Oh what fun it  is  to ride      in  a       one horse o- pen    sleigh!
;.db L|2, b1, b1, L|4, b1, L|2, b1, b1, L|4, b1, L|2, b1, d2, g1, a1, L|8, b1
;;       Jin- gle     bells,   Jin- gle     bells,   jin- gle all the     way.
;.db L|2, c2, c2, c2, c2, c2, b1, b1, L|1, b1, b1, L|2, d2, d2, c2, a1, L|6, g1, L|2, P
;;        Oh what fun it  is  to ride      in  a       one horse o- pen    sleigh!
;.db L|2, d1, b1, a1, g1, d1, d1, P, d1, d1, b1, a1, g1, L|4, e1, L|2, P
;;      Dash-ing thru the snow in    a  one horse o- pen    sleigh;
;.db L|4, e1, L|2, c2, b1, a1, L|8, d2, L|2, e2, d2, c2, a1, L|4, b1, L|2, P
;;       o'er    the fields we      go,    laugh-ing all the     way.
;.db L|4, d1, L|2, b1, a1, g1, L|4, d1, L|2, P, L|4, d1, L|2, b1, a1, g1, L|4, e1, L|2, P
;;      Bells      on bob- tail    ring,             ma-    king spi- rits   bright
;.db e1, e1, c2, b1, a1, d2, d2, d2, L|1, P, d2, L|2, e2, d2, c2, a1, L|4, g1, d1, END
;;  what fun it  is  to ride and sing        a     sleigh-ing song to-   night, oh:

;;"Oh Tannenbaum"
;.equ	SPEED = 12
;.db L|2, c1, L|3, f1, L|1, f1, L|6, f1, L|2, g1, L|3, a1, L|1, a1, L|6, a1
;;        Oh      Tan-     nen-     baum,     oh      Tan-     nen-     baum,
;.db L|2, a1, g1, a1, L|4, ais1, e1, g1, f1, L|2, P
;;       wie gruen sind    dei- ne Blaet-ter!
;.db c2, c2, a1, L|6, d2, L|2, c2, c2, ais1, L|6, ais1, L|2
;; du gruenst nicht   nur     zur  Som- mer-      Zeit,
;.db ais1, ais1, g1, L|6, c2, L|2, ais1, ais1, a1, L|4, a1, L|2, P
;;   nein  auch  im      Win-      ter,  wenn  es     schneit.
;.db L|2, c1, L|3, f1, L|1, f1, L|6, f1, L|2, g1, L|3, a1, L|1, a1, L|6, a1
;;        Oh      Tan-     nen-     baum,     oh      Tan-     nen-     baum,
;.db L|2, a1, g1, a1, L|4, ais1, e1, g1, f1, L|8, P, END
;;       wie gruen sind    dei- ne Blaet-ter!

;;"Stille Nacht"
;.equ	SPEED = 8
;.db L|6, g1, L|2, a1, L|4, g1, L|12, e1, L|6, g1, L|2, a1, L|4, g1, L|12, e1
;;       Sti-      i-       le      Nacht!    Hei-      li-      ge       Nacht!
;.db L|8, d2, L|4, d2, L|12, b1, L|8, c2, L|4, c2, L|12, g1
;;       All-     les     schlaeft;  ein-     sam      wacht
;.db L|8, a1, L|4, a1, L|6, c2, L|2, b1, L|4, a1, L|6, g1, L|2, a1, L|4, g1, L|12, e1
;;       Nur      das     trau-      te      hoch-    hei-      li-      ge       Paar.
;.db L|8, a1, L|4, a1, L|6, c2, L|2, b1, L|4, a1, L|6, g1, L|2, a1, L|4, g1, L|8, e1, L|4, P
;;       Hol-     der      Kna-      be       im      loc-      ki-     gen      Haar,
;.db L|8, d2, L|4, d2, L|6, f2, L|2, d2, L|4, b1, L|12, c2, L|8, e2, L|4, P
;;      Schlaf     in      himm-   lisch-     er        Ru-      uh'!
;.db L|6, c2, L|2, g1, L|4, e1, L|6, g1, L|2, f1, L|4, d1, LEGATO, L|12, c1, L|8, c1, P, P
;;      Schla-     af       in      himm-   lisch-     er                Ruuuuuuuuuh'!
;.db NORMAL, END

;===============================================================================

.macro exactwait
	;wait exactly (lenH:lenL - 18) cycles
	mov	regL, lenL				;1
	mov	regH, lenH				;2
	ldi	temp, 16+18 ;16 pre-loop cycles		;3
	sub	regL, temp				;4
	sbc	regH, zero				;5
	mov	temp, regL				;6
	andi	temp, 0x03 ;(reg % 4)			;7
	sbrc	temp, 0					;8
	rjmp	PC+1 ;2 cycles => 1 additional cycle
	sbrc	temp, 1					;10
	lpm ;3 cycles => 2 additional cycles
	lsr	regH ;(reg / 4)				;12
	ror	regL					;13
	lsr	regH					;14
	ror	regL					;15
	ldi	temp, 1					;16
waitloop: ;loop for (regH:regL - (lenH:lenL % 4) - 1) cycles
	sub	regL, temp
	sbc	regH, zero
	brne	waitloop ;last loop: brne needs only one cycle
.endmacro

;===============================================================================

reset:
	;initialize PORTB
	ldi	temp, 0x1D
	out	DDRB, temp
	ldi	temp, 0x02
	mov	out0, temp
	ldi	temp, 0x03
	mov	out1, temp
	ldi	temp, 0x06
	mov	out2, temp
	out	PORTB, out0
	
	;initialize timer
	ldi	temp, 0x04 ;Clk/256
	out	TCCR0, temp
	ldi	temp, 1<<TOV0
	out	TIFR, temp
	
	;initialize registers
	clr	zero
	ldi	lenL, LOW(50) ;start with supersonic frequency
	ldi	lenH, HIGH(50)
	ldi	ratio, 7 ;normal mode: tone duration is 7/8 of note length
	ldi	length, 4 ;default note length: 1/4
	ldi	tonelen, 1 ;load first note immediately
	ldi	intdiv, 1
	ldi	step, 1
	ldi	temp, 255
	out	TCNT0, temp
	ldi	ZL, LOW(melody*2) ;note pointer
	ldi	ZH, HIGH(melody*2)
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------
	
tone_loop:
	;tone generator loop
	out	PORTB, out1			;18
	exactwait ;wait (LEN - 18) cycles
	in	temp, TIFR			;1
	sbrs	temp, TOV0			;incl. next line: 15 / 3
	rcall	wait10 ;10 cycles + 3 cycles for rcall = 13 cycles
	sbrc	temp, TOV0			;incl. next line: 17 / 17
	rcall	interrupt ;10 cycles + 3 cycles for rcall = 13 cycles
	out	PORTB, out2			;18
	exactwait ;wait (LEN - 18) cycles
	cpi	step, 1				;1
	brsh	pause_loop			;3 (or branch)
	rcall	wait9 ;9 + 3 (rcall) = 12	;15
	rjmp	tone_loop			;17

pause_loop:
	;no tone loop
	out	PORTB, out0
	cpi	step, 0 ;return to tone generator loop?
	breq	tone_loop
	in	temp, TIFR ;timer overflow?
	sbrc	temp, TOV0
	rcall	interrupt
	rjmp	pause_loop
	
;===============================================================================

wait10:
	nop			;    1
wait9:
	nop			;1 / 2
	rjmp	PC+1		;3 / 4
	rjmp	PC+1		;5 / 6
	ret			;9 / 10

;--------------------

interrupt:
	;timer overflow pseudo-interrupt
	dec	intdiv ;divide by SPEED setting		;1
	brne	interrupt_end				;3
	ldi	intdiv, SPEED*SPEEDMUL
	
	;check if a tone or a pause follows
	cpi	ratio, 8 ;no pause in legato mode
	brsh	next
	cpi	step, 1 ;pause has finished, play next note
	brsh	next
	
	;short pause after note (staccato and normal mode)
	ldi	step, 1
	ldi	tonelen, 0 ;pause length = (note length) * (1 - ratio)
	ldi	inttmp, 8
	sub	inttmp, ratio
pauselength_loop:
	add	tonelen, length
	dec	inttmp
	brne	pauselength_loop
	neg	tonelen
	rjmp	interrupt_end
	
next:
	;load next token from flash
	lpm
	ldi	inttmp, 1
	add	ZL, inttmp
	adc	ZH, zero
	mov	inttmp, lpmreg
	cpi	inttmp, 0
	breq	next
	cpi	inttmp, END
	breq	restart_melody
	cpi	inttmp, P
	breq	load_pause
	cpi	inttmp, LEGATO
	brsh	load_mode
	cpi	inttmp, L
	brsh	load_length
	
	;load note
	ldi	step, 0
	mov	regL, ZL ;backup Z pointer
	mov	regH, ZH
	mov	ZL, lpmreg ;load frequency
	ldi	ZH, 0
	lpm
	mov	lenL, lpmreg
	ldi	inttmp, 1
	add	ZL, inttmp
	adc	ZH, zero
	lpm
	mov	lenH, lpmreg
	mov	ZL, regL ;restore Z pointer
	mov	ZH, regH
	ldi	tonelen, 0 ;tone length = (note length) * (ratio)
	mov	inttmp, ratio
tonelength_loop:
	add	tonelen, length
	dec	inttmp
	brne	tonelength_loop
	neg	tonelen

interrupt_end:
	out	TCNT0, tonelen				;4
	ldi	inttmp, 1<<TOV0 ;clear overflow flag	;5
	out	TIFR, inttmp				;6
	ret						;10
	
load_pause:
	;load pause
	ldi	step, 1
	mov	tonelen, length
	lsl	tonelen ;multiply by 8 (ratio = 8/8)
	lsl	tonelen
	lsl	tonelen
	neg	tonelen
	rjmp	interrupt_end

load_length:
	;load note length
	mov	length, lpmreg
	andi	length, 0x1F
	rjmp	next
	
load_mode:
	;set staccato/normal/legato mode
	ldi	ratio, 7 ;normal mode: 7/8 ratio
	cpi	inttmp, STACCATO
	brne	PC+2
	ldi	ratio, 6 ;staccato mode: 3/4 ratio
	cpi	inttmp, LEGATO
	brne	PC+2
	ldi	ratio, 8 ;legato mode: 1/1 ratio
	rjmp	next
	
restart_melody:
	ldi	ZL, LOW(melody*2)
	ldi	ZH, HIGH(melody*2)
	ldi	ratio, 7 ;restore default values
	ldi	length, 4
	;sleep while "Start" input is high
	out	PORTB, out0 ;disable output
	ldi	inttmp, 1<<INTF0 ;enable external interrupt
	out	GIFR, inttmp
	ldi	inttmp, 1<<INT0
	out	GIMSK, inttmp
	ldi	inttmp, 1<<SM | 1<<SE ;enable power-down sleep mode
	out	MCUCR, inttmp
	sleep
	ldi	inttmp, 0
	out	MCUCR, inttmp ;disable sleep mode
	out	GIMSK, inttmp ;disable external interrupt
	rjmp	next
